@@ -59,8 +59,8 @@ attribute_init (GP11Attribute *attr, gulong attr_type,
 	memset (attr, 0, sizeof (GP11Attribute));
 	attr->type = attr_type;
 	attr->length = length;
-	if (value && length) {
-		attr->value = (allocator) (NULL, length);
+	if (value) {
+		attr->value = (allocator) (NULL, length ? length : 1);
 		g_assert (attr->value);
 		memcpy (attr->value, value, length);
 	}
@@ -558,8 +558,8 @@ attribute_init_copy (GP11Attribute *dest, const GP11Attribute *src, GP11Allocato
 	 */
 
 	memcpy (dest, src, sizeof (GP11Attribute));
-	if (src->value && src->length) {
-		dest->value = (allocator) (NULL, src->length);
+	if (src->value) {
+		dest->value = (allocator) (NULL, src->length ? src->length : 1);
 		g_assert (dest->value);
 		memcpy (dest->value, src->value, src->length);
 	}
@@ -629,6 +629,28 @@ gp11_attribute_free (GP11Attribute *attr)
 	}
 }
 
+gboolean
+gp11_attribute_equal (gconstpointer a, gconstpointer b)
+{
+	const GP11Attribute *aa = a;
+	const GP11Attribute *ab = b;
+
+	if (!a && !b)
+		return TRUE;
+	if (!a || !b)
+		return FALSE;
+
+	if (aa->type != ab->type)
+		return FALSE;
+	if (aa->length != ab->length)
+		return FALSE;
+	if (!aa->value && !ab->value)
+		return TRUE;
+	if (!aa->value || !ab->value)
+		return FALSE;
+	return memcmp (aa->value, ab->value, aa->length) == 0;
+}
+
 /**
  * SECTION:gp11-attributes
  * @title: GP11Attributes
@@ -927,7 +949,7 @@ gp11_attributes_new_valist (GP11Allocator allocator, va_list va)
 /**
  * gp11_attributes_at:
  * @attrs: The attributes array.
- * @index: The attribute index to retrieve.
+ * @_index: The attribute index to retrieve.
  * 
  * Get attribute at the specified index in the attribute array.
  * 
@@ -937,12 +959,12 @@ gp11_attributes_new_valist (GP11Allocator allocator, va_list va)
  * Return value: The specified attribute.
  **/
 GP11Attribute*
-gp11_attributes_at (GP11Attributes *attrs, guint index)
+gp11_attributes_at (GP11Attributes *attrs, guint _index)
 {
 	g_return_val_if_fail (attrs && attrs->array, NULL);
-	g_return_val_if_fail (index < attrs->array->len, NULL);
+	g_return_val_if_fail (_index < attrs->array->len, NULL);
 	g_return_val_if_fail (!attrs->locked, NULL);
-	return &g_array_index (attrs->array, GP11Attribute, index);
+	return &g_array_index (attrs->array, GP11Attribute, _index);
 }
 
 static GP11Attribute*
@@ -1337,6 +1359,23 @@ gp11_attributes_unref (GP11Attributes *attrs)
 	}
 }
 
+gboolean
+gp11_attributes_contains (GP11Attributes *attrs, GP11Attribute *match)
+{
+	GP11Attribute *attr;
+	guint i;
+
+	g_return_val_if_fail (attrs && attrs->array, FALSE);
+
+	for (i = 0; i < attrs->array->len; ++i) {
+		attr = gp11_attributes_at (attrs, i);
+		if (gp11_attribute_equal (attr, match))
+			return TRUE;
+	}
+
+	return FALSE;
+}
+
 /* -------------------------------------------------------------------------------------------
  * INTERNAL
  * 
