@@ -29,6 +29,42 @@
 
 #include <string.h>
 
+/**
+ * SECTION:gp11-module
+ * @title: GP11Module
+ * @short_description: A loaded and initialized PKCS#11 module.
+ * 
+ * A GP11Module object holds a loaded PKCS#11 module. A PKCS#11 module is a shared library. 
+ * 
+ * You can load and initialize a PKCS#11 module with the gp11_module_initialize() call. If you already
+ * have a loaded and initialized module that you'd like to use with the various GP11 functions, then 
+ * you can use gp11_module_new(). 
+ */
+
+/**
+ * GP11Module:
+ * 
+ * Holds a loaded and initialized PKCS#11 module.
+ */
+
+/**
+ * GP11ModuleInfo:
+ * @pkcs11_version_major: The major version of the module.
+ * @pkcs11_version_minor: The minor version of the module.
+ * @manufacturer_id: The module manufacturer.
+ * @flags: The module PKCS&num;11 flags.
+ * @library_description: The module description.
+ * @library_version_major: The major version of the library.
+ * @library_version_minor: The minor version of the library.
+ * 
+ * Holds information about the PKCS&num;11 module. 
+ * 
+ * This structure corresponds to CK_MODULE_INFO in the PKCS#11 standard. The 
+ * strings are NULL terminated for easier use. 
+ * 
+ * Use gp11_module_info_free() to release this structure when done with it.
+ */
+
 /*
  * MT safe 
  * 
@@ -64,7 +100,7 @@ typedef struct _GP11ModulePrivate {
 	GStaticMutex mutex;
 	gboolean finalized;
 	GHashTable *open_sessions;
-	gboolean auto_authenticate;
+	gint auto_authenticate;
 } GP11ModulePrivate;
 
 #define GP11_MODULE_GET_DATA(o) \
@@ -438,11 +474,14 @@ gp11_module_get_property (GObject *obj, guint prop_id, GValue *value,
 		g_value_set_pointer (value, gp11_module_get_functions (self));
 		break;
 	case PROP_AUTO_AUTHENTICATE:
-		g_value_set_boolean (value, gp11_module_get_auto_authenticate (self));
+		g_value_set_int (value, gp11_module_get_auto_authenticate (self));
 		break;
 	case PROP_POOL_SESSIONS:
 		g_value_set_boolean (value, gp11_module_get_pool_sessions (self));
 		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, prop_id, pspec);
+		break;
 	}
 }
 
@@ -464,11 +503,14 @@ gp11_module_set_property (GObject *obj, guint prop_id, const GValue *value,
 		data->funcs = g_value_get_pointer (value);
 		break;
 	case PROP_AUTO_AUTHENTICATE:
-		gp11_module_set_auto_authenticate (self, g_value_get_boolean (value));
+		gp11_module_set_auto_authenticate (self, g_value_get_int (value));
 		break;
 	case PROP_POOL_SESSIONS:
 		gp11_module_set_pool_sessions (self, g_value_get_boolean (value));
 		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, prop_id, pspec);
+		break;
 	}
 }
 
@@ -543,27 +585,84 @@ gp11_module_class_init (GP11ModuleClass *klass)
 	klass->authenticate_object = gp11_module_real_authenticate_object;
 	klass->authenticate_slot = gp11_module_real_authenticate_slot;
 
+	/**
+	 * GP11Module:path:
+	 * 
+	 * The PKCS&num;11 module file path. 
+	 * 
+	 * This may be set to NULL if this object was created from an already
+	 * initialized module via the gp11_module_new() function. 
+	 */
 	g_object_class_install_property (gobject_class, PROP_PATH,
 		g_param_spec_string ("path", "Module Path", "Path to the PKCS11 Module",
 		                     NULL, G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
+	/**
+	 * GP11Module:functions:
+	 * 
+	 * The raw PKCS&num;11 function list for the module.
+	 * 
+	 * This points to a CK_FUNCTION_LIST structure. 
+	 */
 	g_object_class_install_property (gobject_class, PROP_FUNCTIONS,
 		g_param_spec_pointer ("functions", "Function List", "PKCS11 Function List",
 		                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
+	/**
+	 * GP11Module:auto-authenticate:
+	 * 
+	 * Whether or not to automatically authenticate token objects that need
+	 * a C_Login call before they can be used.
+	 * 
+	 * The #GP11Module::authenticate-object signal will be fired when an 
+	 * object needs to be authenticated.
+	 */
 	g_object_class_install_property (gobject_class, PROP_AUTO_AUTHENTICATE,
-		g_param_spec_boolean ("auto-authenticate", "Auto Authenticate", "Auto Login to Token when necessary",
-		                      FALSE, G_PARAM_READWRITE));
+		g_param_spec_int ("auto-authenticate", "Auto Authenticate", "Auto Login to Token when necessary",
+		                  0, G_MAXINT, 0, G_PARAM_READWRITE));
 	
+	/**
+	 * GP11Module:pool-sessions:
+	 * 
+	 * Whether or not to pool PKCS&num;11 sessions. When this is set, sessions
+	 * will be pooled and reused if their flags match when gp11_slot_open_session() 
+	 * is called. 
+	 */
 	g_object_class_install_property (gobject_class, PROP_POOL_SESSIONS,
 		g_param_spec_boolean ("pool-sessions", "Pool Sessions", "Pool sessions?",
 		                      FALSE, G_PARAM_READWRITE));
-	
+
+	/**
+	 * GP11Module::authenticate-slot:
+	 * @module: The module
+	 * @slot: The slot to be authenticated.
+	 * @string: A displayable label which describes the object.
+	 * @password: A gchar** where a password should be returned. 
+	 * 
+	 * This signal is emitted when a password is needed to authenticate a PKCS&num;11 
+	 * slot. If the module prompts for passwords itself, then this signal will 
+	 * not be emitted.
+	 * 
+	 * Returns: FALSE if the user cancelled, TRUE if we should proceed.
+	 */
 	signals[AUTHENTICATE_SLOT] = g_signal_new ("authenticate-slot", GP11_TYPE_MODULE, 
 			G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (GP11ModuleClass, authenticate_slot),
 			g_signal_accumulator_true_handled, NULL, _gp11_marshal_BOOLEAN__OBJECT_STRING_POINTER, 
 			G_TYPE_BOOLEAN, 3, GP11_TYPE_SLOT, G_TYPE_STRING, G_TYPE_POINTER);
 
+	/**
+	 * GP11Module::authenticate-object:
+	 * @module: The module.
+	 * @object: The object to be authenticated.
+	 * @label: A displayable label which describes the object.
+	 * @password: A gchar** where a password should be returned.
+	 * 
+	 * This signal is emitted when a password is needed to authenticate a PKCS&num;11
+	 * object like a key. If the module prompts for passwords itself, then this signal will 
+	 * not be emitted.
+	 * 
+	 * Returns: FALSE if the user cancelled, TRUE if we should proceed.
+	 */
 	signals[AUTHENTICATE_OBJECT] = g_signal_new ("authenticate-object", GP11_TYPE_MODULE, 
 			G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (GP11ModuleClass, authenticate_object),
 			g_signal_accumulator_true_handled, NULL, _gp11_marshal_BOOLEAN__OBJECT_STRING_POINTER, 
@@ -668,9 +767,7 @@ gp11_module_initialize (const gchar *path, gpointer reserved, GError **err)
 /**
  * gp11_module_new:
  * @funcs: Initialized PKCS#11 function list pointer
- * @reserved: Extra arguments for the PKCS#11 module, should usually be NULL.
- * @err: A location to store an error resulting from a failed load.
- * 
+ *
  * Create a GP11Module representing a PKCS#11 module. It is assumed that 
  * this the module is already initialized. In addition it will not be 
  * finalized when complete.
@@ -680,6 +777,7 @@ gp11_module_initialize (const gchar *path, gpointer reserved, GError **err)
 GP11Module*
 gp11_module_new (CK_FUNCTION_LIST_PTR funcs)
 {
+	g_return_val_if_fail (funcs, NULL);
 	return g_object_new (GP11_TYPE_MODULE, "functions", funcs, NULL);
 }
 
@@ -885,7 +983,7 @@ gp11_module_get_pool_sessions (GP11Module *self)
 /**
  * gp11_module_set_pool_sessions:
  * @self: The module to set the setting on.
- * @reuse: Whether to reuse sessions or not.
+ * @pool: Whether to reuse sessions or not.
  * 
  * When this is set, sessions will be pooled and reused
  * if their flags match when gp11_slot_open_session() is called.
@@ -919,11 +1017,11 @@ gp11_module_set_pool_sessions (GP11Module *self, gboolean pool)
  * 
  * Return value: Whether auto login or not.
  **/
-gboolean
+gint
 gp11_module_get_auto_authenticate (GP11Module *self)
 {
 	GP11ModulePrivate *pv = lock_private (self);
-	gboolean ret;
+	gint ret;
 	
 	g_return_val_if_fail (pv, FALSE);
 	
@@ -939,7 +1037,7 @@ gp11_module_get_auto_authenticate (GP11Module *self)
 /**
  * gp11_module_set_auto_authenticate:
  * @self: The module to set the setting on.
- * @auto_login: Whether auto login or not.
+ * @auto_authenticate: Whether auto login or not.
  * 
  * When this is set, this slot 
  * will emit the 'authenticate-slot' signal when a session
@@ -947,14 +1045,18 @@ gp11_module_get_auto_authenticate (GP11Module *self)
  * signal when an object requires authintication.
  **/
 void
-gp11_module_set_auto_authenticate (GP11Module *self, gboolean auto_login)
+gp11_module_set_auto_authenticate (GP11Module *self, gint auto_authenticate)
 {
 	GP11ModulePrivate *pv = lock_private (self);
+	
+	/* HACK: Get needed fix around API freeze. */
+	if (auto_authenticate == 1)
+		auto_authenticate = GP11_AUTHENTICATE_TOKENS | GP11_AUTHENTICATE_OBJECTS;
 
 	g_return_if_fail (pv);
 	
 	{
-		pv->auto_authenticate = auto_login;
+		pv->auto_authenticate = auto_authenticate;
 	}
 	
 	unlock_private (self, pv);
@@ -964,14 +1066,13 @@ gp11_module_set_auto_authenticate (GP11Module *self, gboolean auto_login)
 /**
  * gp11_module_enumerate_objects:
  * @self: The module to enumerate objects.
- * @attrs: Attributes that the objects must have, or empty for all objects.
  * @func: Function to call for each object.
  * @user_data: Data to pass to the function.
- * 
+ * @...: The arguments must be triples of: attribute type, data type, value.
+ *
  * Call a function for every matching object on the module. This call may 
  * block for an indefinite period.
  * 
- * The arguments must be triples of: attribute type, data type, value
  * 
  * <para>The variable argument list should contain:
  * 	<variablelist>
@@ -1073,12 +1174,14 @@ gp11_module_enumerate_objects_full (GP11Module *self, GP11Attributes *attrs,
 	slots = gp11_module_get_slots (self, TRUE);
 	
 	for (l = slots; ret && !stop && l; l = g_list_next (l)) {
-		session = gp11_slot_open_session (l->data, CKF_SERIAL_SESSION, &error);
+
+		/* TODO: We really should allow the caller to specify the flags, at least read-write */
+		session = gp11_slot_open_session (l->data, CKF_RW_SESSION | CKF_SERIAL_SESSION, &error);
 		if (!session) {
 			g_return_val_if_fail (error != NULL, FALSE);
 			
 			/* Ignore these errors when enumerating */
-			if (error->code == CKR_USER_PIN_NOT_INITIALIZED) {
+			if (g_error_matches (error, GP11_ERROR, CKR_USER_PIN_NOT_INITIALIZED)) {
 				g_clear_error (&error);
 				
 			} else {
@@ -1116,3 +1219,16 @@ gp11_module_enumerate_objects_full (GP11Module *self, GP11Attributes *attrs,
 	return ret;
 }
 
+/**
+ * GP11ObjectForeachFunc:
+ * @object: The enumerated object.
+ * @user_data: Data passed to enumerate function.
+ * 
+ * This function is passed to gp11_module_enumerate_objects() or a similar function.
+ * It is called once for each object matched. 
+ * 
+ * The GP11Session through which the object is accessible can be retrieved by calling
+ * gp11_object_get_session() on object.
+ * 
+ * Returns: TRUE to continue enumerating, FALSE to stop.
+ */
